from typing import Generic, Optional, TypeVar

T = TypeVar('T')


class MustOpt(Generic[T]):
    def __init__(self):
        self._value: Optional[T] = None
        self._valid: bool = False

    @staticmethod
    def new(value: Optional[T] = None) -> 'MustOpt[T]':
        res = MustOpt()
        res.set(value)
        return res

    def valid(self) -> bool:
        return self._valid

    def must(self) -> T:
        if not self._valid:
            msg = 'Underlying value is not valid'
            raise RuntimeError(msg)

        return self._value

    def set(self, value: Optional[T]):  # noqa: A003
        self._value = value
        self._valid = self._value is not None

    def unset(self):
        self._value = None
        self._valid = False



import unittest

from hamcrest import assert_that, equal_to

from mustopt import MustOpt


class TestMustOpt(unittest.TestCase):
    def test_empty_new_is_invalid(self):
        assert_that(MustOpt.new().valid(), equal_to(False))

    def test_new_from_value_is_valid(self):
        assert_that(MustOpt.new(1).valid(), equal_to(True))

    def test_unset_makes_container_invalid(self):
        val = MustOpt.new(1)
        val.unset()
        assert_that(val.valid(), equal_to(False))

    def test_set_value_makes_container_valid(self):
        val = MustOpt.new()
        val.set(1)
        assert_that(val.valid(), equal_to(True))

    def test_set_none_makes_container_invalid(self):
        val = MustOpt.new()
        val.set(None)
        assert_that(val.valid(), equal_to(False))

    def test_must_from_valid_container_works(self):
        assert_that(MustOpt.new(1).must(), equal_to(1))

    def test_must_from_invalid_container_raises_exception(self):
        try:
            MustOpt.new().must()
        except RuntimeError as e:
            assert_that(str(e), equal_to('Underlying value is not valid'))
